import { ConfigOptions, IConfig, IPlugin } from 'cli-engine-config';
import { args } from 'cli-flags';
import { HTTP } from 'http-call';
import { IFlag } from './flags';
export interface IMockReturn<T extends Command> {
    cmd: T;
    stdout: string;
    stderr: string;
}
export declare type CommandRunFn = <T extends Command>(this: ICommandClass<T>, argv: string[], config: IConfig) => Promise<T>;
export declare type CommandMockFn = <T extends Command>(this: ICommandClass<T>, argv?: string[], config?: ConfigOptions) => Promise<IMockReturn<T>>;
export interface ICommandClass<T extends Command> {
    mock: CommandMockFn;
    run: CommandRunFn;
    new (config: IConfig): T;
}
export declare abstract class Command {
    protected config: IConfig;
    static topic: string;
    static command: string | undefined;
    static description: string | undefined;
    static hidden: boolean;
    static usage: string | undefined;
    static help: string | undefined;
    static aliases: string[];
    static variableArgs: boolean;
    static flags: {
        [name: string]: IFlag<any>;
    };
    static args: args.IArg[];
    static _version: any;
    static plugin: IPlugin | undefined;
    static readonly id: string;
    /**
     * instantiate and run the command setting {mock: true} in the config (shorthand method)
     */
    static mock: CommandMockFn;
    /**
     * instantiate and run the command
     */
    static run: CommandRunFn;
    static buildHelp(config: IConfig): string;
    static buildHelpLine(config: IConfig): [string, string | undefined];
    http: typeof HTTP;
    flags: {
        [name: string]: any;
    };
    argv: string[];
    args: {
        [name: string]: string;
    };
    topic: null;
    command: null;
    description: null;
    hidden: null;
    usage: null;
    help: null;
    aliases: null;
    readonly ctor: typeof Command;
    constructor(config: IConfig);
    init(argv: string[]): Promise<void>;
    /**
     * actual command run code goes here
     */
    abstract run(): Promise<void>;
}
