"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pjson = require('../package.json');
const deps_1 = require("./deps");
class Command {
    constructor(config) {
        this.config = config;
        this.flags = {};
        this.args = {};
        if (deps_1.default.HTTP) {
            this.http = deps_1.default.HTTP.defaults({
                headers: {
                    'user-agent': `${this.config.name}/${this.config.version} (${this.config.platform}-${this.config.arch}) node-${process.version}`,
                },
            });
        }
    }
    static get id() {
        let cmd = [];
        if (this.topic)
            cmd.push(this.topic);
        if (this.command)
            cmd.push(this.command);
        return cmd.join(':');
    }
    static buildHelp(config) {
        let help = new deps_1.default.Help(config);
        return help.command(this);
    }
    static buildHelpLine(config) {
        let help = new deps_1.default.Help(config);
        return help.commandLine(this);
    }
    get ctor() {
        return this.constructor;
    }
    async init(argv) {
        const parse = await deps_1.default.CLIFlags.parse({
            argv,
            args: this.ctor.args || [],
            flags: this.ctor.flags || {},
            strict: !this.ctor.variableArgs,
        });
        this.flags = parse.flags;
        this.argv = parse.argv;
        this.args = parse.args;
    }
}
Command.aliases = [];
Command.variableArgs = false;
Command.args = [];
// tslint:disable-next-line
Command._version = pjson.version;
/**
 * instantiate and run the command setting {mock: true} in the config (shorthand method)
 */
Command.mock = async function (argv = [], config = {}) {
    if (deps_1.default.cli)
        deps_1.default.cli.config.mock = true;
    const cmd = await this.run(argv, deps_1.default.Config.buildConfig(config));
    return {
        cmd,
        stderr: deps_1.default.cli ? deps_1.default.cli.stderr.output : 'cli-ux not found',
        stdout: deps_1.default.cli ? deps_1.default.cli.stdout.output : 'cli-ux not found',
    };
};
/**
 * instantiate and run the command
 */
Command.run = async function (argv = [], config) {
    const cmd = new this(config);
    try {
        await cmd.init(argv);
        await cmd.run();
        if (deps_1.default.cli)
            await deps_1.default.cli.done();
    }
    catch (err) {
        if (deps_1.default.cli)
            deps_1.default.cli.error(err);
        else
            throw err;
    }
    return cmd;
};
exports.Command = Command;
